@@ -1,5 +1,5 @@
 # Package Lingua::DetectCharset
-# Version 1.00
+# Version 1.01
 # Part of "WWW Cyrillic Encoding Suite"
 # Get docs and newest version from
 #	http://www.neystadt.org/cyrillic/
@@ -9,47 +9,53 @@
 # To obtain permision for redistribution or any other usage
 #	contact john@neystadt.org.
 #
+# Portions copyright by 
+#
 # Drop me a line if you deploy this script on tyour site.
 
 package Lingua::DetectCharset;
 
-$VERSION = "1.00";
+$VERSION = "1.01";
 
 use Convert::Cyrillic;
 use Lingua::DetectCharset::StatKoi;
 use Lingua::DetectCharset::StatWin;
+use Lingua::DetectCharset::StatUtf8;
 
 $PairSize = 2;
 $MinRatio = 1.5; # Mark must be in $MinRatio times larger of 
 			# one encoding than another to decide upon, or ENG.
 $DoubtRatio = 1;
-$DoubtLog = 'doubt.txt';
+$DoubtLog = 'DetectCharsetDoubt.txt';
 
 sub Detect {
 	my (@Data) = @_;
 	my ($KoiMark) = GetCodeScore ('Koi', @Data);
 	my ($WinMark) = GetCodeScore ('Win', @Data);
+	my ($Utf8Mark) = GetCodeScore ('Utf8', @Data);
 
-#	print "GetEncoding: Koi8 - $KoiMark, Win - $WinMark\n";
+	# print STDERR "GetEncoding: Koi8 - $KoiMark, Win - $WinMark, Utf8 - $Utf8Mark\n";
 
-	$KoiRatio =  $KoiMark/($WinMark+1);
-	$WinRatio =  $WinMark/($KoiMark+1);
+	$KoiRatio =  $KoiMark/($WinMark+$Utf8Mark+1);
+	$WinRatio =  $WinMark/($KoiMark+$Utf8Mark+1);
+	$Utf8Ratio = $Utf8Mark/($KoiMark+$WinMark+1);
 
 	if ($DoubtLog) {
 		if (($KoiRatio < $MinRatio && $KoiRatio > $DoubtRatio) ||
-			($WinRatio < $MinRatio && $WinRatio > $DoubtRatio)) {
+			($WinRatio < $MinRatio && $WinRatio > $DoubtRatio) ||
+			($Utf8Ratio < $MinRatio && $Utf8Ratio > $DoubtRatio)) {
 				open Log, ">>$DoubtLog";
-				print Log " Koi8 - $KoiMark, Win - $WinMark\n", 
+				print Log " Koi8 - $KoiMark, Win - $WinMark, Utf8 - $Utf8Mark\n", 
 					join ("\n", @Data), "\n\n";
 				close Log;
 		}
 	}
 
-	return 'KOI8' if $KoiRatio > $WinRatio;	# $MinRatio;
-#	return 'WIN'; 				# if $WinRatio > $MinRatio;
+	return 'KOI8' if $KoiRatio > $WinRatio && $KoiRatio > $Utf8Ratio;	# $MinRatio;
+	return 'WIN' if $WinRatio > $Utf8Ratio;
 
 	# We do english, only if no single cyrillic character were detected
-	return 'WIN' if $WinRatio + $KoiRatio > 0;
+	return 'UTF-8' if $WinRatio + $KoiRatio + $Utf8Ratio > 0;
 	return 'ENG';
 }
 
@@ -61,10 +67,15 @@ sub GetCodeScore {
 		$Table = \%Lingua::DetectCharset::StatKoi::StatsTableKoi;
 	} elsif ($Code eq 'Win') {
 		$Table = \%Lingua::DetectCharset::StatWin::StatsTableWin;
+	} elsif ($Code eq 'Utf8') {
+		$Table = \%Lingua::DetectCharset::StatUtf8::StatsTableUtf8Long;
 	} else {
 		die "Don't know $Code!\n";
 	}
 
+	$PairSize = 4 if $Code eq 'Utf8';
+	$PairSize = 2 if $Code ne 'Utf8';
+
 	my ($Mark, $i);
 	for (@Data) {
 		s/[\n\r]//go;
@@ -92,7 +103,7 @@ use Lingua::DetectCharset;
 
 $Charset = Lingua::DetectCharset::Detect ($Buffer); 
 
-The returned $Ecoding is either 'WIN', 'KOI8' or 'ENG'. The last is return when 
+The returned $Charset is either 'WIN', 'KOI8', 'UTF-8' or 'ENG'. The last is return when 
 no single cyrillic token are found in buffer.
 
 =head1 DESCRIPTION
@@ -108,6 +119,7 @@ tests.
 =head1 AUTHOR
 
 John Neystadt <john@neystadt.org>
+Portions by M@kr <http://pub.kem.ru/dev>
 
 =head1 SEE ALSO
 
